/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <libspe2.h>
#include <mars/mars.h>

#define INFO	\
"\
MARS Task Barrier Sample					\n\
------------------------					\n\
This program shows the basic usage of the MARS task barrier	\n\
synchronization between host program and multiple MPU programs. \n\
								\n\
This program creates %d tasks and initializes a task barrier.	\n\
When all %d tasks finish processing, they each notify the	\n\
barrier. Once all tasks notify the barrier and the barrier	\n\
is released, the final task is allowed to process.		\n\
\n", NUM_TASKS, NUM_TASKS

#define NUM_TASKS	10

extern struct spe_program_handle task1_spe_prog;
extern struct spe_program_handle task2_spe_prog;

static struct mars_context mars_ctx;
static struct mars_task_id task1_id[NUM_TASKS];
static struct mars_task_id task2_id;
static struct mars_task_params task_params;
static struct mars_task_args task_args;
static struct mars_task_barrier barrier;

int main(void)
{
	int ret, i;

	printf(INFO);

	ret = mars_initialize(&mars_ctx, NULL);
	if (ret) {
		printf("MARS context initialize failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_task_barrier_initialize(&mars_ctx, &barrier, NUM_TASKS);
	if (ret) {
		printf("MARS task barrier initialize failed! (%d)\n", ret);
		return 1;
	}

	for (i = 0; i < NUM_TASKS; i++) {
		char name[MARS_TASK_NAME_LEN_MAX];
		snprintf(name, MARS_TASK_NAME_LEN_MAX, "Task %d", i);

		task_params.name = (const char *)name;
		task_params.elf_image = task1_spe_prog.elf_image;
		task_params.context_save_size = MARS_TASK_CONTEXT_SAVE_SIZE_MAX;
		ret = mars_task_initialize(&mars_ctx, &task1_id[i], &task_params);
		if (ret) {
			printf("MARS task 1 initialize failed! (%d)\n", ret);
			return 1;
		}

		task_args.type.u64[0] = (uint64_t)(uintptr_t)&barrier;
		ret = mars_task_schedule(&task1_id[i], &task_args, 0);
		if (ret) {
			printf("MARS task 1 schedule failed! (%d)\n", ret);
			return 1;
		}
	}

	task_params.name = "Task Final";
	task_params.elf_image = task2_spe_prog.elf_image;
	task_params.context_save_size = MARS_TASK_CONTEXT_SAVE_SIZE_MAX;
	mars_task_initialize(&mars_ctx, &task2_id, &task_params);
	if (ret) {
		printf("MARS task 2 initialize failed! (%d)\n", ret);
		return 1;
	}

	task_args.type.u64[0] = (uint64_t)(uintptr_t)&barrier;
	ret = mars_task_schedule(&task2_id, &task_args, 0);
	if (ret) {
		printf("MARS task 2 initialize failed! (%d)\n", ret);
		return 1;
	}

	for (i = 0; i < NUM_TASKS; i++) {
		ret = mars_task_wait(&task1_id[i]);
		if (ret) {
			printf("MARS task 1 wait failed! (%d)\n", ret);
			return 1;
		}

		ret = mars_task_finalize(&task1_id[i]);
		if (ret) {
			printf("MARS task 1 finalize failed! (%d)\n", ret);
			return 1;
		}
	}

	ret = mars_task_wait(&task2_id);
	if (ret) {
		printf("MARS task 2 wait failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_task_finalize(&task2_id);
	if (ret) {
		printf("MARS task 2 finalize failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_finalize(&mars_ctx);
	if (ret) {
		printf("MARS context finalize failed! (%d)\n", ret);
		return 1;
	}

	return 0;
}


